@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 #include "canvas.h"
 #include "blockchart.h"
 #include "blockcsh.h"
+#include "block_parser_compile.h"
 
 using namespace block;
 
@@ -44,15 +45,15 @@ std::string LayoutStat::Dump(std::chrono::steady_clock::duration time, size_t no
                                  "(%zu constraints of which mandatory=%zu, removed=%zu, pruned=%zu, rejected=%zu) "
                                  "(%zu variables of which removed=%zu)",
              no_passes, no_blocks, what, (long int)std::chrono::duration_cast<std::chrono::milliseconds>(time).count(),
-             no_constraints, no_must_constraints, no_removed_constraints, no_pruned_constraints, no_rejected_constraints, 
+             no_constraints, no_must_constraints, no_removed_constraints, no_pruned_constraints, no_rejected_constraints,
              no_variables, no_removed_variables);
     return buff;
 }
 
 
-std::unique_ptr<Csh> BlockChart::CshFactory(Csh::FileListProc proc, void *param) const
+std::unique_ptr<Csh> BlockChart::CshFactory(Csh::FileListProc proc) const
 {
-    return std::make_unique<BlockCsh>(proc, param);
+    return std::make_unique<BlockCsh>(proc);
 }
 
 std::map<std::string, std::string> BlockChart::RegisterLibraries() const
@@ -66,7 +67,7 @@ class MyGLPKException
 
 };
 
-static void my_glp_error_hook_function(void *info)
+static void my_glp_error_hook_function(void*)
 {
     throw MyGLPKException();
 }
@@ -82,7 +83,7 @@ BlockChart::BlockChart(FileReadProcedure *p, void *param) : ChartBase(p, param),
     glp_error_hook(&my_glp_error_hook_function, nullptr);
     //Do these before the first input file is added
     parent_style.Empty();
-    parent_name.clear();
+    current_parent = nullptr;
     block_seq = 0;
 }
 
@@ -110,7 +111,7 @@ std::unique_ptr<BlockInstrList> BlockChart::PopContext()
     //the last number, so after 1.2.2 comes 1.3
     ret = std::make_unique<BlockInstrList>();
     if (old_size > MyCurrentContext().numberingStyle.Size())
-        ret->Append(std::make_unique<SetNumbering>(*this, MyCurrentContext().numberingStyle.Size(), 
+        ret->Append(std::make_unique<SetNumbering>(*this, MyCurrentContext().numberingStyle.Size(),
                                                    MyCurrentContext().numberingStyle.Last().increment));
     else
         //We should never get here, but if the length is increasing, we just expand the Numbering list
@@ -125,7 +126,7 @@ bool BlockChart::AddCommandLineArg(const std::string & /*arg*/)
 }
 
 void BlockChart::AddCommandLineOption(const Attribute & a)
-{    
+{
     AddAttribute(a);
 }
 
@@ -139,8 +140,8 @@ bool BlockChart::AddAttribute(const Attribute &a)
         return true;
     }
     if (a.Is("numbering")) {
-        MyCurrentContext().running_style_blocks.AddAttribute(Attribute("number", a), this);
-        MyCurrentContext().running_style_arrows.AddAttribute(Attribute("number", a), this);
+        MyCurrentContext().running_style_blocks.write().AddAttribute(Attribute("number", a), this);
+        MyCurrentContext().running_style_arrows.write().AddAttribute(Attribute("number", a), this);
         return true;
     }
     if (a.Is("conflict_report")) {
@@ -189,9 +190,9 @@ bool BlockChart::AddRunningStyleAttribute(const Attribute &a, EUseKeywords e)
     if (e==0) return true;
     bool ret = false;
     if (USE_KEYWORD_BLOCKS & e)
-        ret |= MyCurrentContext().running_style_blocks.AddAttribute(a, this);
+        ret |= MyCurrentContext().running_style_blocks.write().AddAttribute(a, this);
     if (USE_KEYWORD_ARROWS & e)
-        ret |= MyCurrentContext().running_style_arrows.AddAttribute(a, this);
+        ret |= MyCurrentContext().running_style_arrows.write().AddAttribute(a, this);
     return ret;
 }
 
@@ -220,9 +221,9 @@ bool BlockChart::AddDesignAttribute(const Attribute &a)
         if (NumberingStyleFragment::Parse(this, a.linenum_value.start, a.value.c_str(), nsfs)) {
             int off = MyCurrentContext().numberingStyle.Apply(nsfs);
             if (off > 0) {
-                string msg = "Numbering here is ";
-                msg << off << " levels deep, and you specified more (" << nsfs.size();
-                msg << ") levels of formatting. Ignoring option.";
+                string msg = StrCat("Numbering here is ", std::to_string(off),
+                                    " levels deep, and you specified more (", std::to_string(nsfs.size()),
+                                    ") levels of formatting. Ignoring option.");
                 Error.Error(a, true, msg);
             }
         }
@@ -340,19 +341,42 @@ bool BlockChart::AttributeValues(std::string_view attr, Csh &csh)
 }
 
 
-bool BlockChart::DeserializeGUIState(std::string_view)
+bool BlockChart::DeserializeGUIState(std::string_view s)
 {
+    GUIState state;
+    while (s.size()) {
+        size_t p = s.find('*');
+        if (p==0 || p==s.npos) return false;
+        if (s.size()<p+3) return false;
+        if (s[p+2]!='*') return false;
+        state[std::string(s.substr(0, p))].collapsed = s[p+1]=='1';
+        s.remove_prefix(p+3);
+    }
+    GUI_State = std::move(state);
     return true;
 }
 
 std::string BlockChart::SerializeGUIState() const
 {
-    return std::string();
+    std::string ret;
+    for (auto& [n, c]: GUI_State)
+        ret.append(n).append("*").append(c.collapsed ? "1" : "0").append("*");
+    return ret;
 }
 
 bool BlockChart::ControlClicked(Element *arc, EGUIControlType t)
 {
-    return false;
+    if (!arc) return false;
+    const auto c = arc->GetControls();
+    if (std::ranges::find(c, t)==c.end()) return false;
+    const BlockBlock* const b = dynamic_cast<BlockBlock*>(arc);
+    _ASSERT(b); //only BlockBlocks shall have controls
+    if (!b) return false;
+    switch (t) {
+    case EGUIControlType::EXPAND: GUI_State[b->name_gui_collapse].collapsed = false; return true;
+    case EGUIControlType::COLLAPSE: GUI_State[b->name_gui_collapse].collapsed = true; return true;
+    default: return false;
+    }
 }
 
 bool BlockChart::ApplyForcedDesign(const string & name)
@@ -422,7 +446,7 @@ void BlockChart::AddAttributeListToStyleListWithFilePos(gsl::owner<AttributeList
             Error.Error(*a, false, msg); //we set a->error, but the copies of 'a' in 'attrs' are left unmarked.
         }
     else //no attributes, we are defining empty styles
-        for (auto &s : list) 
+        for (auto &s : list)
             MyCurrentContext().styles[s] = MyCurrentContext().styles.GetStyle(s); //may be default style
     delete al;
 }
@@ -440,7 +464,9 @@ unsigned BlockChart::ParseText(std::string_view input, std::string_view filename
 {
     unsigned ret = current_file = Error.AddFile(filename);
     Progress.RegisterParse(input.length());
-    BlockParse(*this, input.data(), input.length());
+    procedure_parse_helper proc_helper;
+    sv_reader<false> reader(input, current_file);
+    block_compile_parse(*this, proc_helper, reader); //return value ignored
     return ret;
 }
 
@@ -494,7 +520,7 @@ BlockChart::CreateAttribute(std::string_view name, std::string_view value,
                             const FileLineColRange & ln, const FileLineColRange & lv) const
 {
     const std::string str = StringFormat::RemovePosEscapesCopy(value);
-    return new Attribute(name, AlignmentAttr::AttributeNameDir(name) ? value : str, 
+    return new Attribute(name, AlignmentAttr::AttributeNameDir(name) ? value : str,
                          ln, lv);
 }
 
@@ -531,7 +557,7 @@ BlockChart::AutoCreateBlock(std::string_view name, const FileLineColRange & l,
 							const AttributeList *al, const AlignmentAttr * align)
 {
 	if (SkipContent() || !BlockNameToCreatePart(name)) return {};
-	const int shape = MyCurrentContext().running_style_blocks.shape.value_or(-1);
+	const int shape = MyCurrentContext().running_style_blocks.write().shape.value_or(-1);
 	auto b = shape<0 ?
 		std::make_unique<BlockBlock>(*this, EBlockType::Box, l, name) :
 		std::make_unique<BlockBlock>(*this, shape, l, name);
@@ -593,22 +619,26 @@ void BlockChart::StashArrows(const StringWithPosList* l1, ArrowType t, const Str
         ah.bl->remove(nullptr);
     }
 	//If auto define is allowed, we create a block using the default running style (and the shape in it)
-	if (!MyCurrentContext().pedantic.value_or(false))
-		for (auto l :{ l1, l2 })
-			if (l)
-                for (auto &bn: *l) {
-                    std::string_view name = bn.name;
-                    if (BlockNameToCreatePart(name) &&
-                        GetBlockByName(std::string(name), MyCurrentContext().blockname_prefix, true, false)[0]==nullptr) {
-                        auto b = AutoCreateBlock(name, bn.file_pos);
-                        StashedElements.push_back(std::move(b));
-                    }
+    const bool auto_create = !MyCurrentContext().pedantic.value_or(false);
+	for (auto l :{ l1, l2 })
+		if (l) {
+            for (auto &bn: *l)
+                if (std::string_view name = bn.name; name.empty())
+                    Error.Error(bn.file_pos.start, "Block name missing.");
+                else if (auto_create && BlockNameToCreatePart(name) &&
+                         GetBlockByName(std::string(name), MyCurrentContext().parent, true, false)[0] == nullptr) {
+                    auto b = AutoCreateBlock(name, bn.file_pos);
+                    StashedElements.push_back(std::move(b));
                 }
+        }
     if (l1 && l2)
         for (auto &bn1 : *l1)
-            for (auto &bn2 : *l2)
-                StashedElements.Append(CreateArrow(bn1.name.c_str(), t, bn2.name.c_str(),
-                                                   ah, l, bn1.file_pos, bn2.file_pos));
+            if (bn1.name.size())
+                for (auto& bn2 : *l2)
+                    if (bn2.name.size())
+                        StashedElements.Append(CreateArrow(bn1.name, t, bn2.name,
+                                                           ah, l, bn1.file_pos, bn2.file_pos));
+    std::erase_if(StashedElements, [](const std::unique_ptr<BlockInstruction>& i) { return !i->valid; });
     ah.Free();
 }
 
@@ -669,12 +699,12 @@ BlockInstruction *BlockChart::CreateJoin(const FileLineColRange &l, gsl::owner<S
  * It is not const because we may emit errors.*/
 gsl::owner<CopyParseHelper*> BlockChart::ResolveCopyHelper(gsl::owner<CopyParseHelper*> helper)
 {
-    if (helper && helper->block_name) {
+    if (helper && !helper->block_name.had_error && !helper->block_name.empty()) {
         //first try to look up among templates
-        auto i = Templates.find(helper->block_name);
+        auto i = Templates.find(helper->block_name.view());
         if (i==Templates.end())
             helper->block = GetBlockByNameWithError(helper->block_name, helper->block_pos,
-                                                    MyCurrentContext().blockname_prefix,
+                                                    MyCurrentContext().parent,
                                                     "block copy attempt");
         else
             helper->block = i->second.get();
@@ -690,11 +720,7 @@ gsl::owner<CopyParseHelper*> BlockChart::ResolveCopyHelper(gsl::owner<CopyParseH
                 parent_style.Empty();
         } else
             parent_style.Empty();
-        parent_name = helper->copy_name==nullptr || helper->copy_name[0]==0 ?
-                            MyCurrentContext().blockname_prefix :
-                            MyCurrentContext().blockname_prefix.length() ?
-                                StrCat(MyCurrentContext().blockname_prefix, '.', helper->copy_name) :
-                                helper->copy_name;
+        current_parent = FormChildName(GetCurrentPrefix(), helper->copy_name);
     }
     return helper;
 }
@@ -709,32 +735,20 @@ BlockBlock *BlockChart::CreateCopy(gsl::owner<CopyParseHelper*> helper)
         //But CloneAs will do just that (since it is also used for descendants)
         //So Here we manufacture the right alignment: The running style plus any attribute specified by the user
         //This will be applied after the copied alignment, hopefully overwriting it in both dimensions
-        auto use_alignment = std::make_unique<AlignmentAttr>(MyCurrentContext().running_style_blocks.alignment);
+        auto use_alignment = std::make_unique<AlignmentAttr>(MyCurrentContext().running_style_blocks.read().alignment);
         if (helper->align) {
             use_alignment->MergeByDimension(*helper->align);
             delete helper->align;
         }
-        if (helper->copy_name)
-            ret = helper->block->CloneAs(helper->file_pos, helper->copy_name,
-                                         MyCurrentContext().blockname_prefix,
-                                         use_alignment.release(),
-                                         helper->attributes, helper->modifiers);
-        else {
-            const std::string name = helper->block->name_original.length() ?
-                StrCat("copy of ", helper->block->name_original, '@', std::string_view(helper->block_pos.Print()).substr(2)) :
-                StrCat("copy of ", std::string_view(helper->block_pos.Print()).substr(2));
-            ret = helper->block->CloneAs(helper->file_pos, name, //unique name if user specified none
-                                         MyCurrentContext().blockname_prefix,
-                                         use_alignment.release(),
-                                         helper->attributes, helper->modifiers);
-			ret->wildcard_repl = helper->block->wildcard_repl;
-        }
+        ret = helper->block->CloneAs(helper->file_pos, helper->copy_name.empty() ? helper->block->name_original : helper->copy_name.view(),
+                                     GetCurrentPrefix(), use_alignment.release(),
+                                     helper->attributes, helper->modifiers, nullptr);
         //kill parts consumed by Clone()
         helper->align = nullptr;
         helper->attributes = nullptr;
         helper->modifiers = nullptr;
     } //else We got no block. Error already emitted by parser.
-    helper->Free();
+    helper->destroy();
     delete helper;
     return ret;
 }
@@ -769,7 +783,7 @@ void BlockChart::CreateTemplate(gsl::owner<BlockBlock*> block)
  * it makes sure it only passes block names in 'name').
  * The parameter 'align' may contain an alignment attribute set,
  * which is written before the name, like below a b; We apply it
- * before the attributeluist.*/
+ * before the attributelist.*/
 std::unique_ptr<BlockBlock>
 BlockChart::UpdateBlock(std::string_view name, const FileLineColRange & l,
 						const AttributeList *al, const AlignmentAttr * align)
@@ -785,10 +799,10 @@ BlockChart::UpdateBlock(std::string_view name, const FileLineColRange & l,
 		return {};
 	}
 	//Look up the block. If we dont have one with the exact same name, we create one.
-	auto b = GetBlockByName(std::string(name), MyCurrentContext().blockname_prefix, false, false);
+	auto b = GetBlockByName(std::string(name), MyCurrentContext().parent, false, false);
 	if (b[1]) {
 		//More than one block, give error.
-		GetBlockByNameWithError(std::string(name), l.start, MyCurrentContext().blockname_prefix, "here");
+		GetBlockByNameWithError(std::string(name), l.start, MyCurrentContext().parent, "here");
 		return {};
 	}
 	std::unique_ptr<BlockBlock> ret;
@@ -798,9 +812,9 @@ BlockChart::UpdateBlock(std::string_view name, const FileLineColRange & l,
 			ret = AutoCreateBlock(name, l, al, align);
 		else
 			Error.Error(l.start, StrCat("There is no such block named as '", name,
-										MyCurrentContext().blockname_prefix.length() ?
-										"' when looked up from '"+MyCurrentContext().blockname_prefix : "",
-										"'."),
+                                        GetCurrentPrefix().length()
+                                        ? StrCat("' when looked up from '", GetCurrentPrefix())
+                                        : "", "'."),
 						"Use chart option 'pedantic=no;' to automatically create blocks when used first.");
 	} else {
         //just apply attributes
@@ -851,18 +865,21 @@ BlockChart::UpdateBlock(const StringWithPosList * spl, const AttributeList * al,
  *
  * @param [in] n The name of the block: unique name or as the user specifies it in an attr
  *               or arrow specification. May contain a partial full name (the tail of it).
- * @param [in] used_from The prefix at the definition of the block or arrow. This is used
- *             to disambiguite partial full names.
+ * @param [in] used_from The parent block at the definition of the block or arrow. This is used
+ *             to disambiguate partial full names.
  * @param [in] liberal If true, we also hit on any block with this original name - if there
  *                     is only one. Else, we force some prefix match.
  * @param [in] only_valid consider valid blocks.
+ * @param [in] loc You can supply a location in the file, where 'n' is coming from.
+ *                 If there are multiple hits and one of them is at 'loc', we will return
+ *                 *only* that.
  * @returns at most three matching names. */
 std::array<const BlockBlock *, 3>
-BlockChart::GetBlockByName(const std::string &n, std::string_view used_from,
-                           bool liberal, bool only_valid) const
+BlockChart::GetBlockByName(std::string_view n, const Parent& used_from,
+                           bool liberal, bool only_valid, const FileLineCol* loc) const
 {
     std::array<const BlockBlock *, 3> ret = {nullptr, nullptr, nullptr};
-    _ASSERT(n.length());
+    if (n.empty()) return ret;
     //first check if this is a unique name. This is what we replace 'prev', etc to.
     const auto unique = BlockNames_Unique.find(n);
     if (unique!=BlockNames_Unique.end()) {
@@ -881,22 +898,38 @@ BlockChart::GetBlockByName(const std::string &n, std::string_view used_from,
     }
     //If there is only one <used_from>.<n> name exists, that will be the one.
     //This ensures that we can always refer to our siblings via their original names
-    const auto full = BlockNames_Full.equal_range(used_from.length() ? StrCat(used_from, '.', n) : n);
-    if (full.first!=full.second && std::next(full.first)==full.second) {
-        if (!only_valid || full.first->second->valid)
-            ret[0] = full.first->second;
-        return ret;
+    const auto full = BlockNames_Full.equal_range(FormChildName(used_from.GetName(),n));
+    if (full.first != full.second) {
+        if (std::next(full.first) == full.second) {
+            if (!only_valid || full.first->second->valid)
+                ret[0] = full.first->second;
+            return ret;
+        }
+        //If there are multiple ones, we search the one whose parent is 'used_from'
+        if (used_from.AsBlock())
+            for (auto i = full.first; i != full.second; ++i)
+                if (i->second->GetParent() == used_from.AsBlock()) {
+                    if (!only_valid || i->second->valid)
+                        ret[0] = i->second;
+                    return ret;
+                }
     }
-    //OK, no easy find: find all ending in 'n', where the beginning (the part before 'n')
+    //OK, no easy find: 'n' probably has a dot. Find all ending in 'n', where the beginning (the part before 'n')
     //equals to (the beginning of) 'used_from'. Rank them based on the length of the name,
     //the longer the better.
     //now cycle through the full names ending in "."+'n'.
     //Note that BlockNames_Full is sorted taking its strings in reverse
     //from is the first that may end in "."+'n', till is the one beyond the last
     //Note in ASCII '.' < alphanum chars
-    const std::string dotn = "."+n;
+    //If we find any block, where 'used_from' is a parent, we will only search for such blocks.
+    //This is to disambiguate within a block if a block of the same name exists:
+    //E.g.: box A { box X, Y; X->Y; } box A { box X, Y; X->Y; } should have each arrow use the
+    //respective X and Y source and dest even if the full name is A.X and A.Y in both cases.
+    const std::string dotn = "."+ string(n);
     const auto from = BlockNames_Full.lower_bound(dotn);
-    const auto till = BlockNames_Full.lower_bound(StrCat('.', n[0]+1, std::string_view(n.c_str()+1, n.length()-1)));
+    const auto till = BlockNames_Full.lower_bound(StrCat('.', n[0]+1, n.substr(1)));
+    const std::string_view used_from_full_name = used_from.GetName();
+    bool had_one_in_used_from = false; //did we find any blocks that have 'used_from' as parent?
     for (auto i = from; i!=till; i++) {
         //test that n still matches the end
         //E.g., if we have tested B.A, B.C.A, DDDD.A (in this order) for 'dotn'==".A",
@@ -906,15 +939,24 @@ BlockChart::GetBlockByName(const std::string &n, std::string_view used_from,
             break;
         //The length of used_from + length of n shall not be smaller than the length
         //of 'i'. This would be, e.g. 'used_from'="A.B", n="C.D", but i="A.B.X.Y.C.D" - not a match
-        if (i->first.length()>used_from.length()+dotn.length())
+        if (i->first.length()>used_from_full_name.length()+dotn.length())
             continue;
-        //See if the part before 'n' matches the beginning of 'used_from'
+        //See if the part before 'n' matches the beginning of 'used_from_full_name'
         //E.g., the full name must start with 'used_from'
         const ptrdiff_t prefix_len = i->first.length() - dotn.length();
-        if (memcmp(i->first.data(), used_from.data(), prefix_len))
+        if (memcmp(i->first.data(), used_from_full_name.data(), prefix_len))
             continue;
+        const bool i_is_in_used_from = i->second->IsMyParent(used_from.AsBlock());
+        if (i_is_in_used_from && !had_one_in_used_from) {
+            ret = { nullptr, nullptr, nullptr };
+            had_one_in_used_from = true;
+        }
         if (only_valid && !i->second->valid)
             continue;
+        else if (loc && i->second->file_pos.start==*loc)
+            return ret = { i->second, nullptr, nullptr };
+        else if (!i_is_in_used_from && had_one_in_used_from)
+            continue;
         else if (ret[0]==nullptr)
             ret[0] = i->second;
         else if (ret[1]==nullptr)
@@ -927,6 +969,8 @@ BlockChart::GetBlockByName(const std::string &n, std::string_view used_from,
     for (auto i = equal.first; i!=equal.second; i++) {
         if (only_valid && !i->second->valid)
             continue;
+        else if (loc && i->second->file_pos.start==*loc)
+            return ret = { i->second, nullptr, nullptr };
         else if (ret[0]==nullptr)
             ret[0] = i->second;
         else if (ret[1]==nullptr)
@@ -943,26 +987,27 @@ BlockChart::GetBlockByName(const std::string &n, std::string_view used_from,
  * If there are multiple blocks with this (original) name, we emit an error.
  * @param [in] n The name to search for
  * @param [in] l The location of the block name in the file.
- * @param [in] used_from The prefix from which the name shall be looked up.
+ * @param [in] used_from The block from which the name shall be looked up.
  *                       This is kind of the default prefix.
  * @param [in] whattoignore In the text of the error we say we ignore what is in this
  *             string, e.g., "attribute" "arrow", etc.
  * @returns nullptr if no block or multiple blocks by this name.*/
 const BlockBlock *
-BlockChart::GetBlockByNameWithError(const std::string &n, const FileLineCol &l,
-                                    std::string_view used_from, std::string_view whattoignore)
+BlockChart::GetBlockByNameWithError(std::string_view n, const FileLineCol &l,
+                                    const Parent &used_from, std::string_view whattoignore)
 {
-    auto v = GetBlockByName(n, used_from, true, true);
+    auto v = GetBlockByName(n, used_from, true, true, &l);
+    const std::string_view used_from_full_name = used_from.GetName();
     if (v[0]==nullptr) {
-        std::string msg = "Could not find a block named '" + n;
-        if (used_from.length())
-            msg.append(StrCat("' when looked up from prefix '", used_from));
+        std::string msg = "Could not find a block named '" + string(n);
+        if (used_from_full_name.length())
+            msg.append(StrCat("' when looked up from prefix '", used_from_full_name));
         Error.Error(l, StrCat(msg , "'. Ignoring this ", whattoignore, '.'));
         return nullptr;
     } else if (v[1]) {
-        std::string msg = "You have more than one block with name '" + n;
-        if (used_from.length())
-            msg.append(StrCat("' when looked up from prefix '", used_from));
+        std::string msg = "You have more than one block with name '" + string(n);
+        if (used_from_full_name.length())
+            msg.append(StrCat("' when looked up from prefix '", used_from_full_name));
         Error.Error(l, StrCat(msg, "' available and I don't know which one did you mean. Ignoring this ", whattoignore, "."));
         for (unsigned u = 0; u<3 && v[u]; u++)
             Error.Error(v[u]->file_pos.start, l, StrCat("Here is one candidate '", v[u]->name_full, "'."));
@@ -1079,11 +1124,11 @@ Overlaps(iter1 first1, iter1 last1,
                     }
                     //store so that we increase in me first->second
                     if (my_pos[0]<my_pos[1])
-                        ret.push_back({ { xy[0], u, my_pos[0], v, other_pos[0] },
-                        { xy[1], u, my_pos[1], v, other_pos[1] } });
+                        ret.push_back({ { xy[0], u, my_pos[0], v, other_pos[0], {} },
+                        { xy[1], u, my_pos[1], v, other_pos[1], {} } });
                     else
-                        ret.push_back({ { xy[1], u, my_pos[1], v, other_pos[1] },
-                        { xy[0], u, my_pos[0], v, other_pos[0] } });
+                        ret.push_back({ { xy[1], u, my_pos[1], v, other_pos[1], {} },
+                        { xy[0], u, my_pos[0], v, other_pos[0], {} } });
                 }
         }
     //OK, now we have segments of overlap in ret.
@@ -1159,13 +1204,13 @@ std::vector<Arrow*> BlockChart::CheckArrowOverlap()
                     }
                     if (add_start) {
                         PathPos other = same_ends ? (*j)->path_visible.GetStartPos() : (*j)->path_visible.GetEndPos();
-                        contour::CPData cpd{(*i)->path_visible.front().GetStart(), PathPos{0, 0}, other};
+                        contour::CPData cpd{(*i)->path_visible.front().GetStart(), PathPos{0, 0}, other, {}};
                         overlaps.emplace(overlaps.begin(), cpd, cpd);
                     }
                     if (add_end) {
                         PathPos me = (*i)->path_visible.GetEndPos();
                         PathPos other = same_ends ? (*j)->path_visible.GetEndPos() : (*j)->path_visible.GetStartPos();
-                        contour::CPData cpd{(*i)->path_visible.back().GetEnd(), me, other};
+                        contour::CPData cpd{(*i)->path_visible.back().GetEnd(), me, other, {}};
                         overlaps.emplace_back(cpd, cpd);
                     }
                     //OK, now the front/back of 'overlaps' is a zero length overlap segment if the arrows
@@ -1212,7 +1257,7 @@ std::vector<Arrow*> BlockChart::CheckArrowOverlap()
                                     [end](auto &a) {return a.first->adjusted[end ^ !a.second.same_ends]; }))
                 continue;
             //If all the touchpoints are just endpoints, dont do a thing.
-            if (std::ranges::none_of(pA->overlaps[end], 
+            if (std::ranges::none_of(pA->overlaps[end],
                                      [](auto &a) {return a.second.type>=Arrow::Overlap::SAME_TANGENT; })) {
                 //mark these arrow ends as done
                 for (auto &a : pA->overlaps[end])
@@ -1312,8 +1357,7 @@ std::vector<Arrow*> BlockChart::CheckArrowOverlap()
 }
 
 
-void BlockChart::CompleteParse(bool autoPaginate, bool addHeading,
-                               XY pageSize, bool fitWidth, bool collectLinkInfo)
+void BlockChart::CompleteParse(bool, bool, XY, bool, bool)
 {
     Canvas canvas(Canvas::Empty::Query);
     pageBreakData.clear();
@@ -1333,7 +1377,7 @@ void BlockChart::CompleteParse(bool autoPaginate, bool addHeading,
 	Progress.RegisterBulk(ESections::DRAW, 1);
     Progress.StartSection(ESections::POSTPROCESS);
 	Numbering number;
-	Blocks.PostParseProcess(canvas, false, number); //BlockNames filled in here
+	Blocks.PostParseProcess(canvas, nullptr, number); //BlockNames filled in here
     Blocks.FinalizeLabel(canvas);
     Blocks.ResolveAlignmentAttributes(nullptr, nullptr, nullptr, nullptr);
     Blocks.style.alignment.Empty();
@@ -1378,7 +1422,7 @@ void BlockChart::CompleteParse(bool autoPaginate, bool addHeading,
             Contour block;
             for (unsigned end = 0; end<2; end++)
                 for (auto &o: pA->overlaps[end])
-                    if (o.first->path.size()) 
+                    if (o.first->path.size())
                         block += o.first->GetMyBlockArea(my_dist);
             pA->Layout(canvas, std::move(block));
         }
@@ -1400,6 +1444,8 @@ void BlockChart::CompleteParse(bool autoPaginate, bool addHeading,
         if (pageBreakData.size()==0)
             pageBreakData.emplace_back(XY(0, 0), total.Spans()); //we *must* have one page
         bkFill.emplace_back(Blocks.style.fill, total);
+        //Prune GUI states: remove GUI commands, referring to no element
+        std::erase_if(GUI_State, [](const GUIState::value_type& s) { return !s.second.element_exists; });
     }
     Error.Sort();
 }
