@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-	Copyright (C) 2008-2022 Zoltan Turanyi
+	Copyright (C) 2008-2023 Zoltan Turanyi
 	Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -23,8 +23,6 @@
 #if !defined(CANVAS_H)
 #define CANVAS_H
 
-#define _CRT_NONSTDC_NO_DEPRECATE //For visual studio to allow strdup
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -70,6 +68,8 @@ EmbedChartData Select(FILE *in, const LanguageCollection &languages,
 } //namespace pngutil
 
 extern std::mutex cairo_text_mutex;
+extern const bool do_not_draw_fonts; ///<Set via environment variables, used to generate images with no fonts in testing
+
 
 ////////////////////Helpers//////////////////////////////////
 inline void thread_safe_cairo_show_text(cairo_t *cr, const char *utf8)
@@ -77,7 +77,7 @@ inline void thread_safe_cairo_show_text(cairo_t *cr, const char *utf8)
 #ifdef WIN32
     std::lock_guard<std::mutex> lock(cairo_text_mutex);
 #endif // CAIRO_HAS_WIN32_SURFACE
-    if (cr)
+    if (cr && !do_not_draw_fonts)
         cairo_show_text(cr, utf8);
 }
 
@@ -86,7 +86,7 @@ inline void thread_safe_cairo_text_path(cairo_t *cr, const char *utf8)
 #ifdef WIN32
     std::lock_guard<std::mutex> lock(cairo_text_mutex);
 #endif // CAIRO_HAS_WIN32_SURFACE
-    if (cr)
+    if (cr && !do_not_draw_fonts)
         cairo_text_path(cr, utf8);
 }
 
@@ -165,8 +165,8 @@ namespace msc {
 }
 
 class PptFile;
-/** A class encapsulating a cairo drawing conext.
- * Besides relaying drawing calls to the cario context Canvas has the following functionality.
+/** A class encapsulating a cairo drawing context.
+ * Besides relaying drawing calls to the cairo context Canvas has the following functionality.
  * - Create and manage various target surfaces (PDF, PNG, EPS, EMF, WMF, recording surface)
  *   including writing to files.
  * - Handle capabilities missing in certain target types (dashed lines, gradient fill,
@@ -243,7 +243,7 @@ public:
      *             effect.
      * @return true if this is a dummy canvas, is already closed or failed to open well.
      *         For non-dummy canvases it ASSERTs that we can do graphics. */
-    bool cannot_draw(bool assert_if_candraw_false = true) const noexcept {
+    bool cannot_draw(bool assert_if_candraw_false [[maybe_unused]] = true) const noexcept {
         if (dummy) return true;
         if (!does_graphics()) return true;
         _ASSERT(candraw || !assert_if_candraw_false);
@@ -256,7 +256,7 @@ protected:
     bool     white_background;        /** Draw a white background before chart drawing.*/
     double   fake_scale;              /** Final rendering should be scaled like this. Used for WMF, since WMF coordinates are 16-bit integers. We are better off if the chart is scaeld up (less rounding), but not more than 32K.*/
     double 	 fallback_resolution;     /** The resolution of the fallback images for vector surfaces.*/
-    bool     needs_dots_in_corner;    /** Draw a dot in upperleft and lowerright corner. Needed for EMF/WMF for the output to include the white space around the chart.*/
+    bool     needs_dots_in_corner;    /** Draw a dot in upper left and lower right corner. Needed for EMF/WMF for the output to include the white space around the chart.*/
     bool     use_text_path;           /** Use cairo_text_path() instead of cairo_show_text(). I am not sure we have any backend needing this.*/
     bool     use_text_path_rotated;   /** Use cairo_text_path() instead of cairo_show_text() for rotated text. WMF cannot display text if the transformation matrix is rotated. */
     int      individual_chars_path;   /** 0: off, 1: Each character is drawn one by one, and not in a single call to cairo_show_text(); 2: non-ascii chars are shown with cairo_text_path(). WMF does not scale text nicely, this is a workaround. */
@@ -297,9 +297,9 @@ protected:
     const bool        external_surface;   ///<True if we got the surface in the constructor externally (no need to destroy it at closure)
     const double      copyrightTextHeight;///<The height of the copyright text
     const XY          user_scale;         ///<Stores the scale user specified at construction
-    Block             raw_page_clip;      ///<Used with fix-page outoput only. Contains the raw page size in points (pixels) minus margins. All zero if not a fixed page output.
-    const int         h_alignment;        ///<Used with fix-page outoput only. Contains -1/0/+1 for left/center/right page alignment.
-    const int         v_alignment;        ///<Used with fix-page outoput only. Contains -1/0/+1 for up/center/down page alignment.
+    Block             raw_page_clip;      ///<Used with fix-page output only. Contains the raw page size in points (pixels) minus margins. All zero if not a fixed page output.
+    const int         h_alignment;        ///<Used with fix-page output only. Contains -1/0/+1 for left/center/right page alignment.
+    const int         v_alignment;        ///<Used with fix-page output only. Contains -1/0/+1 for up/center/down page alignment.
     /** @} */
 
     /** @name Logic
@@ -350,11 +350,11 @@ friend class msc::Pipe;  //for exotic line joints
     /** Fill an area with faked gradient */
     void fakeRadialGrad(ColorType from, ColorType to, const XY &s, double outer_radius, double inner_radius,
                         unsigned steps, bool rectangle, double rad_from=0, double rad_to=2*M_PI);
-    /** Stroke a single (non-doble or triple), which may be dashed - consider `fake_dash`*/
+    /** Stroke a single (non-double or triple), which may be dashed - consider `fake_dash`*/
     void singleLine(const Block &, const LineAttr &line);
-    /** Stroke a single (non-doble or triple), which may be dashed - consider `fake_dash`*/
+    /** Stroke a single (non-double or triple), which may be dashed - consider `fake_dash`*/
     void singleLine(const Path&, const LineAttr &line);
-    /** Stroke a single (non-doble or triple), which may be dashed - consider `fake_dash`*/
+    /** Stroke a single (non-double or triple), which may be dashed - consider `fake_dash`*/
     void singleLine(const Contour&, const LineAttr &line);
     /** @} */
 
@@ -410,8 +410,8 @@ public:
 public:
     /** Set the fallback image resolution.*/
     void SetFallbackImageResolution(double res) {if (error.empty()) cairo_surface_set_fallback_resolution(surface, res/fake_scale, res/fake_scale);}
-    /** Switches to the second drawing situation allowing to draw header (inserted by e.g., automatic pagination) and footer (copyrgight text).*/
-    void PrepareForHeaderFoorter();
+    /** Switches to the second drawing situation allowing to draw header (inserted by e.g., automatic pagination) and footer (copyright text).*/
+    void PrepareForHeaderFooter();
     /** Instruct to add the chart text in an iTXt chunk to a PNG file at the end.
      * The chart_text is expected to remain valid until CloseOutput().
      * Returns false if we are not a PNG output or in error.*/
@@ -436,9 +436,9 @@ public:
     /** True if we should avoid transparency when drawing */
     bool AvoidTransparency() const {return avoid_transparency;}
 
-    /** Adds a generic shape if we do not do graphichs, but collect shapes. */
+    /** Adds a generic shape if we do not do graphics, but collect shapes. */
     bool Add(GenericShape&& shape);
-    /** Adds several generic shapes if we do not do graphichs, but collect shapes. */
+    /** Adds several generic shapes if we do not do graphics, but collect shapes. */
     bool AddMore(std::ranges::range auto&& shapes);
 
     /** Set color, line with and dash */
