@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -55,7 +55,7 @@ std::string VersionText(char a, char b, char c)
 
 
 /** Reads a text file from `in`.
- * If the file is non empty, we enforce trailing newline (as per posix)
+ * If the file is non empty, we enforce trailing newline (as per POSIX)
  * (This is to handle trailing contexts for strings ending in one dot
  * just before EOF.)
  * @param [in] in The file to read from.
@@ -95,26 +95,27 @@ std::string ReadFile(FILE *in, bool till_zero)
 //and then a 0x1 again and then a 0x1 separated list of load data in the format <name>:<load_data>
 LoadDataSet SplitLoadData(std::string *load_data, MscError *Error, const FileLineCol *opt_pos)
 {
-    std::map<std::string, std::string> ldset;
+    LoadDataSet ldset;
     if (load_data && load_data->size()) {
+        std::string_view ld = *load_data;
         if (load_data->front()!='\x1') {
-            //load data fromat version 0
+            //load data format version 0
             ldset["signalling"] = *load_data;
         } else {
             size_t pos1 = 1;
-            size_t pos = load_data->find_first_of(1, pos1);
-            int version = atoi(load_data->substr(pos1, pos-pos1).c_str());
+            size_t pos = ld.find_first_of(1, pos1);
+            int version = to_int(ld.substr(pos1, pos-pos1));
             if (version<=1) {
-                for (; pos<load_data->length(); /*nope*/) {
+                for (; pos<ld.length(); /*nope*/) {
                     pos++;
-                    pos1 = load_data->find_first_of(1, pos);
-                    size_t colon = load_data->find_first_of(':', pos);
-                    if (colon > load_data->length()) {
+                    pos1 = ld.find_first_of(1, pos);
+                    size_t colon = ld.find_first_of(':', pos);
+                    if (colon > ld.length()) {
                         if (Error && opt_pos)
                             Error->Warning(*opt_pos, "Error in load data file. Clearing past load data and will regenerate.");
                         break;
                     }
-                    ldset[load_data->substr(pos, colon-pos)] = load_data->substr(colon+1, pos1-colon-1);
+                    ldset[string(ld.substr(pos, colon-pos))] = ld.substr(colon+1, pos1-colon-1);
                     pos = pos1;
                 }
             } else if (Error && opt_pos)
@@ -143,7 +144,7 @@ Examples::Load(const std::vector<std::string> &example_dirs,
                OpenNamedFileFunction* file_open_proc) {
     if (example_dirs.empty()) return {0,{}};
     const Csh* csh = nullptr;
-    for (const std::string& lname : L->GetLanguages())
+    for (std::string_view lname : L->GetLanguages())
         if (const LanguageData* l = L->GetLanguage(lname))
             if (l->pCsh && l->pCsh->file_list_proc) {
                 csh = l->pCsh.get();
@@ -155,7 +156,7 @@ Examples::Load(const std::vector<std::string> &example_dirs,
     for (const std::string& example_dir : example_dirs) {
         if (example_dir.empty()) continue;
         std::vector<std::string> example_files =
-            csh->file_list_proc(example_dir, {}, csh->file_list_proc_param);
+            csh->file_list_proc(example_dir, {});
         std::ranges::sort(example_files);
         for (const std::string& e: example_files) {
             if (e.empty()) continue;
@@ -258,13 +259,13 @@ std::vector<Example*> Examples::Filter(std::string_view search, const LanguageDa
     return ret;
 }
 
-/** Checks if a command-line argument is acceptable for at least one of the languages 
- * or if it is a valid command-line option.*/
-bool IsValidChartSpecificCommandLineArg(const LanguageCollection& languages, const std::string &arg) {
-    for (const std::string& l : languages.GetLanguages())
+/** For a command-line argument returns the languages for which it is a valid language-specific CLI option .*/
+std::vector<std::string_view> IsValidChartSpecificCommandLineArg(const LanguageCollection& languages, const std::string &arg) {
+    std::vector<std::string_view> ret;
+    for (std::string_view l : languages.GetLanguages())
         if (auto pChart = languages.GetLanguage(l)->create_chart(nullptr, nullptr))
-            if (pChart->AddCommandLineArg(arg)) return true;
-    return false;
+            if (pChart->AddCommandLineArg(arg)) ret.push_back(l);
+    return ret;
 }
 
 /* Indicate to the user that we are reading from stdin.
@@ -285,12 +286,12 @@ std::string version(const LanguageCollection &languages)
     char buff[2000];
     snprintf(buff, sizeof(buff),
         "Msc-generator %s (using %s)\n"
-        "Copyright (C) 2008-2022 Zoltan Turanyi\n"
+        "Copyright (C) 2008-2023 Zoltan Turanyi\n"
         "Msc-generator comes with ABSOLUTELY NO WARRANTY.\n"
         "This is free software, and you are welcome to redistribute it under certain\n"
         "conditions; type `msc-gen -l' for details.\n",
         VersionText(LIBMSCGEN_MAJOR, LIBMSCGEN_MINOR, LIBMSCGEN_SUPERMINOR).c_str(),
-        languages.GetLibraries().c_str());
+        languages.GetLibraryNames().c_str());
     return buff;
 }
 /** Print program usage and return. */
@@ -303,13 +304,13 @@ static void usage()
 "\n"
 "Program information:\n"
 " -h, --help  Display this help and exit.\n"
-" -l          Display program licence and exit.\n"
+" -l          Display program license and exit.\n"
 " --version   Display version information and exit.\n"
 "\n"
 "Operation mode selection:\n"
 " --gui       Using this mode pops up a GUI for interactive editing of a chart.\n"
 "             Some options apply in GUI mode, such as -S to start an empty chart\n"
-"             of a given type; 'infile' to open a file; --no-designs, -D to\n"
+"             of a given type; 'infile' to open a file; --nodesigns, -D to\n"
 "             control design libraries; -F and -L to control the default font.\n"
 " --tei-mode  Using this option puts msc-generator into text editor integration\n"
 "             mode. In this mode msc-gen can produce color syntax highlighting,\n"
@@ -363,7 +364,7 @@ static void usage()
 "             Load file containing additional chart design definitions.\n"
 "             You can have multiple of this option to load several design\n"
 "             files, after the default ones. These files are loaded even if\n"
-"             --no-designs is specified.\n"
+"             --nodesigns is specified.\n"
 " -e          If output type is PNG, specifying this will embed the chart\n"
 "             text into the PNG file as an iTXt chunk. In case of multiple\n"
 "             pages (and multiple output files) all will have it embedded.\n"
@@ -383,7 +384,7 @@ static void usage()
 "             Margins are to be specified in inches (number only) or in\n"
 "             centimeters, if appended with 'cm' (no spaces). The default\n"
 "             margin is half inches everywhere.\n"
-" --no-designs\n"
+" --nodesigns\n"
 "             If you specify this no design files will be loaded (except the\n"
 "             ones you specify with -D). This is useful to increase performance\n"
 "             when you do not use them anyway.\n"
@@ -397,8 +398,8 @@ static void usage()
 " -p[=<page size]\n"
 "             Full-page output. (PDF or PPT only, automatically implied for PPT)\n"
 "             In this case the chart is drawn on fixed-size pages (following\n"
-"             pagination) with one pixel equalling to 1/72 inches for PDF and\n"
-"             1/96 inches for PPT. If a chart page is larger than a physcal\n"
+"             pagination) with one pixel equaling to 1/72 inches for PDF and\n"
+"             1/96 inches for PPT. If a chart page is larger than a physical\n"
 "             page it is simply cropped with a warning. Setting the scale with\n"
 "             the -s option enables zooming. Page size can be set to ISO sizes\n"
 "             from A0 to A6, and to US sizes, such as letter, legal, ledger and\n"
@@ -452,29 +453,37 @@ static void usage()
 " --force-mscgen\n"
 "             Forces the chart to be interpreted in mscgen mode. Note that many\n"
 "             Msc-generator attributes, commands and keywords are still\n"
-"             recognized. This setting makes conflicting syntax be intrepreted\n"
+"             recognized. This setting makes conflicting syntax be interpreted\n"
 "             as mscgen would do. Without this switch Msc-generator uses the\n"
 "             mscgen mode only if the chart starts with the text 'msc {'.\n"
 " --prevent-mscgen\n"
 "             Prevents the chart to be interpreted in mscgen mode. Note that\n"
 "             some mscgen attributes and symbols are still recognized. This\n"
-"             setting makes conflicting syntax be intrepreted as Msc-generator\n"
+"             setting makes conflicting syntax be interpreted as Msc-generator\n"
 "             would do. Without this switch Msc-generator uses the mscgen mode\n"
 "             if the chart starts with the text 'msc {'.\n"
 " -Wno-mscgen Disables warnings for deprecated constructs kept only for\n"
 "             backwards compatibility with mscgen. Has no effect with\n"
 "             --force-mscgen, in that case no such warnings are emitted anyway.\n"
+"\n"
+"Environment variables:\n"
+"  MSCGEN_FONT: can be used to define the default font similar to the -F option.\n"
+"  MSCGEN_FONT_LANG: can be used to define the default language for fontconfig\n"
+"                    based systems, similar to the -L option.\n"
+"  MSCGEN_NOFONT: When set (to any value), Msc-generator draws no text to any\n"
+"                 output (but considers the place of the text at layout as\n"
+"                 normally. Useful for testing.\n"
 "\n");
 }
 
 
 
 
-/** return program licence as a string. */
-std::string licence(const LanguageCollection &languages) {
+/** return program license as a string. */
+std::string license(const LanguageCollection &languages) {
     std::string my =
 "Msc-generator, a chart renderer.\n"
-"Copyright (C) 2008-2022 Zoltan Turanyi\n"
+"Copyright (C) 2008-2023 Zoltan Turanyi\n"
 "Distributed under GNU Affero General Public License.\n"
 "\n"
 "Msc-generator is free software: you can redistribute it and/or modify\n"
@@ -489,9 +498,9 @@ std::string licence(const LanguageCollection &languages) {
 "\n"
 "You should have received a copy of the GNU Affero General Public License\n"
 "along with Msc-generator.  If not, see <http://www.gnu.org/licenses/>.\n";
-    for (auto &[lib, lic] : languages.libraries)
+    for (auto &[lib, lic] : languages.GetLibraries())
         if (lic.empty()) continue;
-        else my.append("\n\n").append(lib).append(" licence:\n").append(lic);
+        else my.append("\n\n").append(lib).append(" license:\n").append(lic);
     return my;
 }
 
@@ -501,7 +510,7 @@ std::string licence(const LanguageCollection &languages) {
  * @param [in] languages The languages supported by the build.
  * @param [in] design_files A list of <name,content> pairs of design library files,
  *                          which shall be parsed before the input file. Any error
- *                          occuring in them will be also reported. We include the
+ *                          occurring in them will be also reported. We include the
  *                          design library for all languages, do_main needs to parse
  *                          only the ones relevant for the language selected by the user.
  *                          If 'content' is empty, we use OpenNamedFile('name') to load it.
@@ -524,8 +533,8 @@ std::string licence(const LanguageCollection &languages) {
  *   or a GUIInit, which means the GUI needs to be displayed.*/
 std::variant<int, GUIInit>
 do_main(const std::vector<std::string> &args,
-    LanguageCollection &languages,
-    std::vector<std::pair<std::string, std::string>> &design_files,
+    LanguageCollection &&languages,
+    std::vector<std::pair<std::string, std::string>> &&design_files,
     OpenNamedFileFunction OpenNamedFile,
     Chart::FileReadProcedure file_read_proc, void *file_read_proc_param,
     string csh_textformat,
@@ -564,7 +573,7 @@ do_main(const std::vector<std::string> &args,
     std::string             oForcedLanguage;
     std::string             oForcedDesign;
     bool                    oNoCopyright = false;
-    std::vector<std::string>oChartArguments;
+    std::map<std::string, std::vector<std::string_view>> oChartArguments; //maps CLI arguments to the languages supporting them as language-specific CLI option.
     std::vector<std::string>oChartOptions;
     bool                    oPaths = true;
     bool                    oTEIMode = false;
@@ -574,7 +583,7 @@ do_main(const std::vector<std::string> &args,
     std::optional<bool>     oPedantic;
     OUnsignedPair           oEmbed;   //If the user says -T embed:slide:chart, these will contain 'slide' and 'chart' (both default to 0 as in 'insert')
     OUnsignedPair           oExtract; //If the user specified a filename ending in ":<num>:<num> this contains the two numbers.
-          
+
 
     if (const char* const font1 = getenv("MSCGEN_FONT")) oFont = font1;
     else if (const char* const font2 = getenv("CHARTGEN_FONT")) oFont = font2;
@@ -585,6 +594,7 @@ do_main(const std::vector<std::string> &args,
     const FileLineCol opt_pos(Error.AddFile("[options]"), 0, 0);
     const FileLineColRange opt_pos_range(opt_pos, opt_pos);
     bool show_usage = false;
+    std::vector<std::string_view> tmp_langs;
 
     for (auto i=args.begin(); i!=args.end(); i++) {
         if (i->length()==0) continue;
@@ -630,7 +640,7 @@ do_main(const std::vector<std::string> &args,
                     Error.Error(opt_pos, "Invalid page size. Should be one of the ISO A-series, such as 'A4p' or 'A3l', or 'letter', 'legal', 'ledger', 'tabloid'. "
                                 "You can also use 'widescreen_l', 'on_screen_4_3_l', 'on_screen_16_9_l' or 'on_screen_16_10_l' suitable for PPT output (use _p for portrait). "
                                 "Using 'A4p' as default (or widescreen_l for PPT).");
-            } 
+            }
             oUseWidthAttr = false;
         } else if (i->length()>=3 && i->at(0) == '-' && (i->at(1) == 'v' || i->at(1) == 'h') &&
                    i->at(2) == 'a') {
@@ -652,14 +662,14 @@ do_main(const std::vector<std::string> &args,
             if (at==nullptr)
                 Error.Error(opt_pos, "Invalid margin selector. Use one of '-ml', '-mr', '-mu' or '-md' for left, right, upper or lower margins, respectively.  Ignoring this.");
             else if (i->at(3) != '=')
-                Error.Error(opt_pos, "After the marging option, I need a number with no spaces, like '-mu=1.2cm'. Ignoring this.");
+                Error.Error(opt_pos, "After the margin option, I need a number with no spaces, like '-mu=1.2cm'. Ignoring this.");
             else {
                 double mul = 72; //pixels per unit. Default is inch.
                 char *buff = (char*)malloc(i->length());
                 buff[0] = 0;
                 double res;
                 if (sscanf(i->c_str()+4, "%lf%s", &res, buff)==0)
-                    Error.Error(opt_pos, "After the marging option, I need a number with no spaces, like '-mu=1.2cm'. Ignoring this.");
+                    Error.Error(opt_pos, "After the margin option, I need a number with no spaces, like '-mu=1.2cm'. Ignoring this.");
                 else {
                     if (!strcmp("cm", buff)) mul = PT_PER_CM;
                     else if (buff[0] && strcmp("in", buff))
@@ -781,7 +791,7 @@ do_main(const std::vector<std::string> &args,
                     if (oTextOutType.length()) {
                         if (oTextOutType.front()!=':')
                             embed_problem:
-                            Error.FatalError(opt_pos, 
+                            Error.FatalError(opt_pos,
                                              "Strange output type after '-T'.",
                                             "Use 'embed:<slide>:<chart>' to update an existing chart, 'embed:<slide>' to insert a new one or just 'embed' to insert it on a new slide.");
                         else {
@@ -856,7 +866,7 @@ do_main(const std::vector<std::string> &args,
                  }
             }
         } else if (*i == "-l") {
-            std::cout<<licence(languages);
+            std::cout<<license(languages);
             return EXIT_SUCCESS;
         } else if (*i == "-h" || *i=="--help") {
             usage();
@@ -896,10 +906,9 @@ do_main(const std::vector<std::string> &args,
             oChartOptions.push_back(i->substr(6));
         } else if (i->starts_with("--design=")) {
             oForcedDesign = i->substr(9);
-        } else if (i->length()>=2 && i->at(0) == '-' 
-                   && IsValidChartSpecificCommandLineArg(languages, *i)) {
-            //may be a chart-specific attribute, test if we accept it
-            oChartArguments.push_back(*i); 
+        } else if (i->length()>=2 && i->at(0) == '-' &&
+                   (tmp_langs = IsValidChartSpecificCommandLineArg(languages, *i)).size()) { //may be a chart-specific attribute, test if we accept it
+            oChartArguments[std::move(*i)] = std::move(tmp_langs);
         } else if (i->at(0) == '-' && i->size()>1) {
             Error.Error(opt_pos, "Unrecognized option: '"+*i+"'. Ignoring it.");
             show_usage = true;
@@ -944,23 +953,40 @@ do_main(const std::vector<std::string> &args,
     }
 
     //Check if the input filename ends with :<num>:<num>
-    if (oInputFile.length()) 
+    if (oInputFile.length())
         if (const size_t pos2 = oInputFile.find_last_of(':'); pos2!=string::npos) {
-            oExtract.emplace(0, 0);
             if (pos2==0)
                 Error.FatalError(opt_pos, "I am confused by an input filename starting with a colon: '"+oInputFile+"'.");
-            else if (const size_t pos1 = oInputFile.find_last_of(':', pos2-1); pos1!=string::npos) {
-                if (from_chars(oInputFile.substr(pos1+1, pos2-pos1-1), oExtract->first) ||
-                    from_chars(oInputFile.substr(pos2+1), oExtract->second))
-                    Error.FatalError(opt_pos, "Could not parse slide number after input file '"+oInputFile+"'.");
-                else
+            //Note: valid Windows paths may contain *one* colon (after the drive letter or server name)
+            //It is possible that the colon is followed by a number as filename, as "C:123", but we
+            //decide to interpret that as a slide number. In the case of "C:123" we will fail to open "C:"
+            //and get an error allowing the user to adjust the path, e.g., to "C:.\123"
+            else if (const size_t pos1 = oInputFile.find_last_of(':', pos2 - 1); pos1 != string::npos) {
+                //we have at least two colons. The text after the last one must be a number
+                if (unsigned last_num; from_chars(std::string_view(oInputFile).substr(pos2 + 1), last_num))
+                    //E.g, "C:file.txt:aaa
+                    Error.FatalError(opt_pos, "Could not parse slide number after input file '" + oInputFile + "'.");
+                else if (unsigned first_num; from_chars(std::string_view(oInputFile).substr(pos1 + 1, pos2 - pos1 - 1), first_num)) {
+                    //The thing between the two colons is not a number
+                    if (std::string_view(oInputFile).substr(0, pos1).find(':')!=std::string_view::npos) //there is a third colon before pos2 - that may be part of the filename
+                        //E.g., "C:file.txt:aaa:123"
+                        Error.FatalError(opt_pos, "Could not parse slide number after input file '" + oInputFile + "'.");
+                    else { //assume the text between the first and second colon it is part of the filename.
+                        //E.g., "C:file.txt:123"
+                        oExtract.emplace(last_num, 0);
+                        oInputFile.erase(pos2);
+                    }
+                } else {
+                    //Both are valid numbers, e.g., "filename.txt:1:2" or "C:file.txt:1:2"
+                    oExtract.emplace(first_num, last_num);
                     oInputFile.erase(pos1);
-            } else {
-                if (from_chars(oInputFile.substr(pos2+1), oExtract->first))
-                    Error.FatalError(opt_pos, "Could not parse slide number after input file '"+oInputFile+"'.");
-                else
-                    oInputFile.erase(pos2);
-            }
+                }
+            } else if (unsigned num; !from_chars(std::string_view(oInputFile).substr(pos2 + 1), num)) {
+                //valid number after the single colon: interpret as slide number
+                //E.g: "file.txt:1", but also "C:123"
+                oExtract.emplace(num, 0);
+                oInputFile.erase(pos2);
+            } //else if not a num, we assume it is part of the filename, e.g.: "C:file.txt"
         }
 
     //Finished parsing the arguments
@@ -1009,7 +1035,7 @@ do_main(const std::vector<std::string> &args,
             oForceUTF8 = false;
         }
     }
-    if (oExtract && type!="ppt") 
+    if (oExtract && type!="ppt")
         Error.Error(opt_pos, "Only 'pptx' files may have a :<slide>:<chart> specifier after the input filename. Ignoring this.");
 
     if (oUseStdIn && oInputFile.size()) {
@@ -1022,15 +1048,15 @@ do_main(const std::vector<std::string> &args,
             Error.FatalError(opt_pos, "Don't use --tei-mode and --gui together.");
             oGUI = false;
         } else {
-            if (oUseStdIn) 
+            if (oUseStdIn)
                 Error.FatalError(opt_pos, "GUI cannot read from stdin. Do not use '--gui' and '-' together.");
             if (oChartArguments.size()==1)
                 Error.FatalError(opt_pos, "Cannot apply chart arguments when invoking the GUI. "
-                                          "Retry without the '"+oChartArguments.front()+"' option.");
+                                          "Retry without the '"+oChartArguments.begin()->first+"' option.");
             else if (oChartArguments.size()>1)
                 Error.FatalError(opt_pos, "Cannot apply chart arguments when invoking the GUI. "
                                           "Retry without the following options: "
-                                          + PrintStrings(oChartArguments));
+                                          + PrintStrings(oChartArguments | std::ranges::views::keys));
             if (oFont.size())
                 Error.FatalError(opt_pos, "Cannot set font from the command-line when invoking the GUI. "
                                           "Retry without the '-F "+oFont+"' option.");
@@ -1055,13 +1081,13 @@ do_main(const std::vector<std::string> &args,
                 Error.FatalError(opt_pos, "Sorry, the GUI can only work with UTF-8 files. Retry without --utf16.");
             if (oPedantic)
                 Error.Error(opt_pos, "--pedantic cannot be used with --gui: set pedantic from the Settings. Ignoring --pedantic.");
-            if (oExtract) 
+            if (oExtract)
                 Error.Warning(opt_pos, "Ignoring <slide> and <chart> setting after the input filename"); //TODO: open GUI with these
             if (oLoadDesigns)
                 for (auto &p : design_files) {
                     //see if the design file is of known type
                     size_t pos = p.first.find_last_of('.');
-                    if (pos != string::npos && languages.LanguageForExtension(p.first.substr(pos+1)).size()) {
+                    if (pos != string::npos && languages.LanguageForExtension(std::string_view(p.first).substr(pos+1)).size()) {
                         // remove path if the user wanted so
                         std::string fn = p.first;
                         if (!oPaths) {
@@ -1080,7 +1106,7 @@ do_main(const std::vector<std::string> &args,
                     }
                 }
             std::erase_if(design_files, [](auto &p) { return p.second.empty(); });
-            languages.CompileDesignLibs(design_files);
+            languages.CompileDesignLibs(std::move(design_files));
         }
     } else if (!oTEIMode) {
         if (oInputFile.empty() && !oUseStdIn) {
@@ -1154,7 +1180,7 @@ do_main(const std::vector<std::string> &args,
         else {
             std::vector<PptFile::ChartInPPT> charts;
             std::string err = PptFile::parse_charts(ZipArchive::FromFile(oInputFile), charts);
-            if (err.size()) 
+            if (err.size())
                 Error.FatalError(opt_pos, err);
             else if (charts.empty())
                 Error.FatalError(opt_pos, "The file '"+oInputFile+"' contains no charts embedded by Msc-generator.");
@@ -1162,7 +1188,7 @@ do_main(const std::vector<std::string> &args,
                 std::optional<size_t> idx;
                 if (!oExtract || oExtract->first==0) {
                     if (charts.size()>1)
-                        Error.FatalError(opt_pos, "The file '"+oInputFile+"' contains " + std::to_string(charts.size()) 
+                        Error.FatalError(opt_pos, "The file '"+oInputFile+"' contains " + std::to_string(charts.size())
                                          + " charts embedded by Msc-generator.",
                                          "Specify the slide number after the filename, like '"+oInputFile+":5'.");
                     else
@@ -1189,7 +1215,7 @@ do_main(const std::vector<std::string> &args,
                     } else {
                         if (oExtract->second==0) {
                             if (charts.size()>1)
-                                Error.FatalError(opt_pos, "The file '"+oInputFile+"' contains " + std::to_string(charts.size()) 
+                                Error.FatalError(opt_pos, "The file '"+oInputFile+"' contains " + std::to_string(charts.size())
                                                  + " embedded charts on slide "+std::to_string(oExtract->first) + ".",
                                                  "Specify the chart number after the slide number, like '"+oInputFile+":"
                                                  + std::to_string(oExtract->first) + ":2'.");
@@ -1211,7 +1237,7 @@ do_main(const std::vector<std::string> &args,
                     already_has_input = true;
                 } else
                     _ASSERT(Error.hasFatal());
-            }  
+            }
         }
     }
 
@@ -1223,7 +1249,7 @@ do_main(const std::vector<std::string> &args,
         if (!oPaths)
             Error.RemovePathFromErrorMessages();
         std::cerr << Error.Print(oWarning, oTechnicalInfo);
-        if (show_usage) 
+        if (show_usage)
             std::cerr<<"Use 'msc-gen --help' for full command-line and options syntax."<<std::endl;
         std::cerr << "Bailing out." << std::endl;
         return EXIT_FAILURE;
@@ -1274,7 +1300,7 @@ do_main(const std::vector<std::string> &args,
             std::cerr << Error.Print(oWarning, oTechnicalInfo);
         }
         MscInitializeCshAppearanceList();
-        return GUIInit{languages, design_files, file_read_proc, file_read_proc_param,
+        return GUIInit{std::move(languages), file_read_proc, file_read_proc_param,
                        OpenNamedFile, CopyrightText, oInputFile, oInputFile, type, load_data};
     }
 
@@ -1300,7 +1326,7 @@ do_main(const std::vector<std::string> &args,
         for (auto &p : design_files) {
             //see if the design file is of our type
             size_t pos = p.first.find_last_of('.');
-            if (pos != string::npos && lang->HasExtension(p.first.substr(pos+1))) {
+            if (pos != string::npos && lang->HasExtension(std::string_view(p.first).substr(pos+1))) {
                 // remove path if the user wanted so
                 std::string fn = p.first;
                 if (!oPaths) {
@@ -1323,9 +1349,9 @@ do_main(const std::vector<std::string> &args,
         }
 
     //Add Chart arguments
-    for (auto &arg : oChartArguments)
+    for (auto &[arg, langs] : oChartArguments)
         if (arg.length() && !chart->AddCommandLineArg(arg)) {
-            chart->Error.Error(opt_pos, "Unrecognized option: "+arg+". Ignoring it.");
+            chart->Error.Error(opt_pos, "Option: "+arg+" is only valid for "+PrintStrings(langs)+" chart(s). Ignoring it.");
             show_usage = true;
         }
     if (oForcedDesign.size() && !chart->ApplyForcedDesign(oForcedDesign)) {
@@ -1339,7 +1365,7 @@ do_main(const std::vector<std::string> &args,
             chart->AddCommandLineOption(Attribute(arg, std::string_view("yes"), opt_pos_range, opt_pos_range));
         else
             //..or another option with a value
-            chart->AddCommandLineOption(Attribute(arg.substr(0, eq), arg.substr(eq+1), opt_pos_range, opt_pos_range));
+            chart->AddCommandLineOption(Attribute(std::string_view(arg).substr(0, eq), std::string_view(arg).substr(eq+1), opt_pos_range, opt_pos_range));
 
     //Add font
     if (oFont.length() || oLang.length())
@@ -1376,7 +1402,7 @@ do_main(const std::vector<std::string> &args,
         case Canvas::PPT:
             oOutputFile.append(".pptx"); break;
         default:
-            assert(0);
+            _ASSERT(0);
         }
     } else if (oOutputFile=="-") {
         //this means standard output
@@ -1487,10 +1513,10 @@ do_main(const std::vector<std::string> &args,
         lang->pCsh->pShapes = &chart->Shapes; //contains all the shapes from the design libs
         //read the design libs
         if (oLoadDesigns) {
-            for (const auto &p : design_files) {
+            for (auto &p : design_files) {
                 size_t pos = p.first.find_last_of('.');
-                if (pos != string::npos && lang->HasExtension(p.first.substr(pos+1)))
-                    lang->pCsh->ParseDesignText(p.second, -1);
+                if (pos != string::npos && lang->HasExtension(std::string_view(p.first).substr(pos+1)))
+                    lang->pCsh->ParseDesignText(std::move(p.second), -1);
             }
             lang->pCsh->CleanupAfterDesignlib();
         }
@@ -1588,7 +1614,7 @@ do_main(const std::vector<std::string> &args,
                 break; //will stop and emit errors
             }
             CshListType prev_csh = std::move(lang->pCsh->CshList);
-            lang->pCsh->ParseText(input, fHint, chart->GetPedantic());
+            lang->pCsh->ParseText(std::string(input), fHint, chart->GetPedantic());
             string output;
             constexpr int scheme = 1;
             if (oCsh)
@@ -1810,7 +1836,7 @@ do_main(const std::vector<std::string> &args,
             FILE* const fout = oOutputFile.length() ? OpenNamedFile(oOutputFile.c_str(), true, true) : stdout;
             chart->labelData.sort([](const LabelInfo &a, const LabelInfo &b) {return a.coord.y.from<b.coord.y.from; });
             for (const auto &l : chart->labelData) {
-                //check page number, step until upper left corner of labe is on a page
+                //check page number, step until upper left corner of label is on a page
                 unsigned p = 0;
                 while (chart->GetPageNum() > p &&
                        !Block(chart->GetPageData(p)->xy, chart->GetPageData(p)->xy + chart->GetPageData(p)->wh)
@@ -1860,10 +1886,10 @@ do_main(const std::vector<std::string> &args,
             if (oEmbed) {
                 //The total chart size with all headers, footers, etc. We assume we draw the whole chart.
                 unsigned page_idx = oEmbed->first, ch_idx = oEmbed->second;
-                EmbedChartData data{.chart_type= lang->GetName(), .chart_text=std::move(input), .chart_size=chart->GetCanvasSize()};
+                EmbedChartData data{.chart_type=lang->GetName(), .chart_text=std::move(input), .chart_size=chart->GetCanvasSize()};
                 //'input' is invalid from here
-                std::string error = 
-                    PptFile::add_image(ZipArchive::FromFile(saved_ppt_outfile), oOutputFile, page_idx, ch_idx, 
+                std::string error =
+                    PptFile::add_image(ZipArchive::FromFile(saved_ppt_outfile), oOutputFile, page_idx, ch_idx,
                                        std::move(data), OpenNamedFile);
                 if (error.size())
                     chart->Error.FatalError(FileLineCol(), "Could not write to PPT: "+error);
@@ -1885,7 +1911,7 @@ fatal:
     if (!oPaths)
         chart->Error.RemovePathFromErrorMessages();
     std::cerr << chart->Error.Print(oWarning, oTechnicalInfo);
-    if (show_usage) 
+    if (show_usage)
         std::cerr<<"Use 'msc-gen --help' for full command-line and options syntax."<<std::endl;
 
     if (chart->Error.hasFatal()) {
