@@ -69,20 +69,21 @@ TextureID TextureFromSurface(cairo_surface_t*);
 void LoadFonts();
 std::map<std::string, std::string> RegisterLibrariesGUI();
 void GUIBeep();
-std::vector<std::string> FileListProc(std::string_view prefix_, std::string_view included_from_, void* app);
+std::vector<std::string> FileListProc(std::string_view prefix_, std::string_view included_from_);
 std::optional<std::string> ShowDocumentation(const std::string& doc_name);///< Open the named doc, or return an error string
+std::optional<std::string> ShowLink(std::string_view url); ///<Open the web browser with this URL (in UTF-8)
 /** The formats we are interested in on the clipboard. */
 enum class ClipboardFormat {
     Art_GVML,    ///<Zipped XML containing something copied from Office
     Escaped_Text ///<Plain text starting with "Msc-generator~|"
 };
-/** Returns the content of the clipboard (if interesting for us). 
- * (2 formats are interesting, see 'ClipboardFormat'.) Any value 
- * is only returned once, that is, after we have returned the content, 
- * subsequent calls to this function will return the empty string 
+/** Returns the content of the clipboard (if interesting for us).
+ * (2 formats are interesting, see 'ClipboardFormat'.) Any value
+ * is only returned once, that is, after we have returned the content,
+ * subsequent calls to this function will return the empty string
  * unless new data of interesting type is copied to the clipboard.*/
 std::pair<ClipboardFormat, std::string> GetClipboard();
-/** Place this to the clipboard. 
+/** Place this to the clipboard.
  * @params [in] format What is the format of 'content' we shall use when placin to clipboard.
  * @params [in] content The content to place to the clipboard.
  * @params [in] type For Art_GVML, this contains the human readable chart type. Ignored for Escaped_Text.
@@ -90,7 +91,7 @@ std::pair<ClipboardFormat, std::string> GetClipboard();
 bool CopyToClipboard(ClipboardFormat format, const std::string& content, std::string_view type = {});
 
 struct GUIReturn {
-    enum Action { 
+    enum Action {
         None,      ///<Normal window
         Exit,      ///<Exit the program
         FullScreen,///<Make the window full-screen (if not yet so)
@@ -125,6 +126,9 @@ struct GUIWindowGeometry {
         cx = std::min(cx, sx);
         cy = std::min(cy, sy);
     }
+    GUIWindowGeometry scale_by(double dpi_mul) const noexcept {
+        return { int(x*dpi_mul), int(y*dpi_mul), int(cx*dpi_mul), int(cy*dpi_mul) };
+    }
 };
 
 /** Initialize the GUI. Call this before a call to DoGui()
@@ -138,7 +142,7 @@ struct GUIWindowGeometry {
  * @param [in] docs The short name of the documentation formats available.
  *             A call to ShowDocumentation() shall supply one of these values.
  * @returns the loaded window positions*/
-GUIWindowGeometry InitGUI(const GUIInit& init, const std::string& settings_dir, const std::string& current_dir,
+GUIWindowGeometry InitGUI(const GUIInit& init, float dpi_mul_, const std::string& settings_dir, const std::string& current_dir,
                           const std::vector<std::string>& example_dir = {}, std::vector<string>&& docs = {});
 
 /** Called before each ImGUI Frame.
