@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -116,7 +116,8 @@ struct GraphDrawingPrimitive
         label(std::move(l)), sx(_sx), dx(_dx), y(_y) {}
     void ShiftBy(const XY &) noexcept;
     Contour Cover(const ShapeCollection &) const; ///<Returns the cover of the drawing primitive
-    void Draw(Canvas &, const ShapeCollection &) const; ///<Actually draws the drawing primitive onto 'canvas'
+    void CollectIsMapElements(Canvas& canvas, Chart& chart) const { label.CollectIsMapElements(chart.ismapData, canvas, chart.Shapes, sx, dx, y); }
+    void Draw(Canvas&, const ShapeCollection&) const; ///<Actually draws the drawing primitive onto 'canvas'
 };
 
 /** A list of drawing primitives */
@@ -130,7 +131,7 @@ protected:
     DrawingList drawingList;  ///<The drawing elements emitted by graphviz. Used only if we rely on the graphviz rendering engine and do not draw ourselves.
 public:
     GraphElement(GraphStyle::EGraphElementType t) : style(EStyleType::ELEMENT, t) {}
-    std::string name;  ///<The name of the graph, cluser, node (or the key of the edge)
+    std::string name;  ///<The name of the graph, cluster, node (or the key of the edge)
     StyleCoW<GraphStyle> style;  ///<The collection of attributes applicable to us.
     bool AddAttribute(const Attribute &a, Chart *chart); ///<Add an Attribute to us.
     /** Adds all our attributes to us. Also Sets label.*/
@@ -145,8 +146,10 @@ public:
     void AddAttributesToGraph(void *obj) const;
     virtual void ShiftBy(const XY &) override;
     virtual void PostPosProcess(Canvas &, Chart *ch) override;
-    /** Draw the list of drawing primitives we have receied.*/
+    /** Draw the list of drawing primitives we have received.*/
     void DrawAll(Canvas &canvas, const ShapeCollection &shapes) const { for (auto &p: drawingList) p.Draw(canvas, shapes); }
+    void CollectIsMapElements(Canvas& canvas) final { _ASSERT(0); /*Call GraphCollectIsMapElements instead */ }
+    virtual void GraphCollectIsMapElements(Canvas& canvas, Chart &chart) const { for (auto& p : drawingList) p.CollectIsMapElements(canvas, chart); }
     /** Draw this graph element (parts with this pass) */
     virtual void Draw(Canvas &canvas, const ShapeCollection &shapes, EGraphDrawPass) const = 0;
 };
@@ -155,10 +158,10 @@ public:
  * It is able to manage descendants of SubGraph, GraphNode and GraphEdge, if they have
  * constructor of the following syntax (one for defining at top level and one
  * for adding it to a subgraph).
- * - SubGraph(Graph &graph, const char *name, bool is_cluster, ...);
- * - SubGraph(SubGraph *parent, const char *name, bool is_cluster, ...);
- * - GraphNode(Graph &graph, const char *name, ...);
- * - GraphNode(SubGraph &parent, const char *name, ...);
+ * - SubGraph(Graph &graph, std::string_view name, bool is_cluster, ...);
+ * - SubGraph(SubGraph *parent, std::string_view name, bool is_cluster, ...);
+ * - GraphNode(Graph &graph, std::string_view name, ...);
+ * - GraphNode(SubGraph &parent, std::string_view name, ...);
  * Or two versions for GraphEdge based on how to handle port string memory ownership
  * - GraphEdge(GraphNode *_1, GraphNode *_2, const std::string &p1, const std::string &p2, const FileLineColRange &pos1, const FileLineColRange &pos2, ...);
  * - GraphEdge(GraphNode *_1, GraphNode *_2, std::string &&p1, std::string &&p2, const FileLineColRange &pos1, const FileLineColRange &pos2, ...);
@@ -192,13 +195,13 @@ public:
     Block boundingBox;            ///<The bounding box after layout.
 
     /** @name variables used during parsing, pseudo-rendering, layouting
-     * These make Graph non-reetrant. But hey, the whole graphviz library
+     * These make Graph non-reentrant. But hey, the whole graphviz library
      * is not reentrant.
      * @{ */
-    static Graph* _current;           ///<A static variable pointing to the Graph currently under parsing, layout or pseudo-rendering. Error messages and rendered drawing primitives are inserted into this.
-    Canvas *_canvas = nullptr;           ///<The canvas during pseudo-rendering.
-    std::string _err_msg_accum;       ///<We collect multi-segment DOT error messages in this.
-    Chart *_chart = nullptr;             ///<The chart to add errors to during layout or parsing.
+    static Graph* _current;      ///<A static variable pointing to the Graph currently under parsing, layout or pseudo-rendering. Error messages and rendered drawing primitives are inserted into this.
+    Canvas *_canvas = nullptr;   ///<The canvas during pseudo-rendering.
+    std::string _err_msg_accum;  ///<We collect multi-segment DOT error messages in this.
+    Chart *_chart = nullptr;     ///<The chart to add errors to during layout or parsing.
     static int DotError(char *msg);
     /** @} */
 
@@ -217,7 +220,7 @@ public:
      * elsewhere in a subgraph, we allocate a new one. (Hopefully mirroring
      * how graphviz handles this.)*/
     template <class SG, typename ...Args>
-    SubGraph *CreateSubGraph(const char *name, bool cluster, bool *created, Args &&... args);
+    SubGraph *CreateSubGraph(std::string_view name, bool cluster, bool *created, Args &&... args);
     /** Create a subgraph of your choice (see notes for class Graph) and add it
      * to the graph subgraph 'g'.
      * Returns false in *created if non-null and if we have found this subgraph
@@ -226,14 +229,14 @@ public:
      * elsewhere, we allocate a new one. (Hopefully mirroring
      * how graphviz handles this.)*/
     template <class SG, typename ...Args>
-    SubGraph *CreateSubGraph(SubGraph *g, const char *name, bool cluster, bool *created, Args &&... args);
+    SubGraph *CreateSubGraph(SubGraph *g, std::string_view name, bool cluster, bool *created, Args &&... args);
 
     /** Create a node of your choice (see notes for class Graph) and add it
      * to the graph subgraph 'g'.
      * Returns false in *created if non-null and if we have found this node
      * already added (by name). In this case we return the one found.*/
     template <class Node, typename ...Args>
-    GraphNode *CreateNode(const char *name, bool *created, Args &&... args);
+    GraphNode *CreateNode(std::string_view name, bool *created, Args &&... args);
     /** Create a node of your choice (see notes for class Graph) and add it
      * at the top level to the graph.
      * Returns false in *created if non-null and if we have found this node
@@ -243,7 +246,7 @@ public:
      * except from 'g' or subgraphs of 'g'. Hopefully this mimics what
      * graphviz is doing. (One node can be in one visible subgraph, at most.*/
     template <class Node, typename ...Args>
-    GraphNode *CreateNode(SubGraph *g, const char *name, bool *created, Args &&... args);
+    GraphNode *CreateNode(SubGraph *g, std::string_view name, bool *created, Args &&... args);
     /** Create an edge of your choice (see notes for class Graph) and add it
      * to the graph.
      * If an edge already exists between these nodes, we return it if the graph is
@@ -266,12 +269,10 @@ public:
     void AddAttrName(GraphStyle::EGraphElementType t, const GraphStyle &s);
     void AddAttrName(GraphStyle::EGraphElementType t, const std::string &attr_name);
 
-    GraphNode *GetNode(const std::string &name) const; ///<Return the node of a given name (if found)
-    GraphNode *GetNode(const char *name) const;        ///<Return the node of a given name (if found)
-    SubGraph *GetSubGraph(const std::string &name) const; ///<Return the subgraph of a given name (if found)
-    SubGraph *GetSubGraph(const char *name) const; ///<Return the subgraph of a given name (if found)
+    GraphNode *GetNode(std::string_view name) const;    ///<Return the node of a given name (if found)
+    SubGraph *GetSubGraph(std::string_view name) const; ///<Return the subgraph of a given name (if found)
     GraphEdge *GetFirstEdge(const GraphNode *_1, const GraphNode *_2) const;
-    GraphElement *GetElement(void *in_gv);                 ///<Return the node, edge or subgraph from their pointer in cgraph (if found)
+    GraphElement *GetElement(void *in_gv);              ///<Return the node, edge or subgraph from their pointer in cgraph (if found)
     SubGraph *GetNodeFromVisibleSubgraph(GraphNode *n);
     bool RemoveEmptySubGraphs();
 
@@ -280,6 +281,7 @@ public:
     void PseudoRenderFrom(Chart*chart);
     void PostPosProcess(Canvas &, Chart *ch) override;
     void AdjustBkArea(const Range& total_x) noexcept;
+    void GraphCollectIsMapElements(Canvas& canvas, Chart& chart) const override;
     void Draw(Canvas& canvas, const ShapeCollection& shapes, EGraphDrawPass) const override;
     std::optional<BackgroundFill> GetBkFill() const noexcept;
 };
@@ -296,12 +298,12 @@ protected:
     virtual void AddToGvGraph() const; ///<Create a counterpart for us in cgraph. Relies on parent to be already there
     virtual void ClearGv() const { in_gv = nullptr; for (auto &s: subgraphs) s->ClearGv(); }
 public: //XXX should be protected, but make_unique needs them.
-    SubGraph(Graph &g, const char *n, bool c) : GraphElement(c ? GraphStyle::CLUSTER : GraphStyle::SUBGRAPH), graph(g), parent(nullptr), cluster(c) { name = n ? n : ""; }
-    SubGraph(SubGraph *g, const char *n, bool c) : GraphElement(c ? GraphStyle::CLUSTER : GraphStyle::SUBGRAPH), graph(g->graph), parent(g), cluster(c) { name = n ? n : ""; }
+    SubGraph(Graph &g, std::string_view n, bool c) : GraphElement(c ? GraphStyle::CLUSTER : GraphStyle::SUBGRAPH), graph(g), parent(nullptr), cluster(c) { name = n; }
+    SubGraph(SubGraph* g, std::string_view n, bool c) : GraphElement(c ? GraphStyle::CLUSTER : GraphStyle::SUBGRAPH), graph(g->graph), parent(g), cluster(c) { name = n; }
 public:
-    Graph &graph;                     ///<The graph we belong to.
+    Graph &graph;                 ///<The graph we belong to.
     SubGraph *const parent;       ///<Our parent subgraph, nullptr if we are directly in a graph.
-    const bool cluster;               ///<True, if we are a visible subgraph
+    const bool cluster;           ///<True, if we are a visible subgraph
     NPtrList<GraphNode> nodes;    ///<The nodes mentioned directly in us.
     NPtrList<SubGraph> subgraphs; ///<We own only the directly included subgraphs
 
@@ -312,11 +314,9 @@ public:
     void Remove(GraphNode *n);
     SubGraph* GetNodeFromVisibleSubgraph(GraphNode *n);
     bool RemoveEmptySubGraphs();
-    GraphNode *GetNode(const std::string &name) const; ///<Return the node of a given name (if found)
-    GraphNode *GetNode(const char *name) const;        ///<Return the node of a given name (if found)
-    SubGraph *GetSubGraph(const std::string &name); ///<Return the subgraph of a given name (if found)
-    SubGraph *GetSubGraph(const char *name); ///<Return the subgraph of a given name (if found)
-    SubGraph *GetSubGraph(Agraph_t *); ///<Return the subgraph of a given pointer in cgraph (if found)
+    GraphNode *GetNode(std::string_view name) const; ///<Return the node of a given name (if found)
+    SubGraph *GetSubGraph(std::string_view name);    ///<Return the subgraph of a given name (if found)
+    SubGraph *GetSubGraph(Agraph_t *);     ///<Return the subgraph of a given pointer in cgraph (if found)
     GraphElement *GetElement(void *in_gv); ///<Return the node, edge or subgraph from their pointer in cgraph (if found)
     NPtrList<GraphNode> GetAllNodes() const;
     virtual void ShiftBy(const XY &) override;
@@ -341,8 +341,8 @@ protected:
     virtual void ClearGv() const { in_gv = nullptr; }
     void SetLabelFromName();
 public: //XXX These should be protected, but make_unique needs them.
-    GraphNode(Graph &g, const char *n) : GraphElement(GraphStyle::NODE), graph(g) { if (n) name = n; SetLabelFromName(); }
-    GraphNode(SubGraph &g, const char *n) : GraphElement(GraphStyle::NODE), graph(g.graph) { if (n) name = n; SetLabelFromName();}
+    GraphNode(Graph &g, std::string_view n) : GraphElement(GraphStyle::NODE), graph(g) { name = n; SetLabelFromName(); }
+    GraphNode(SubGraph &g, std::string_view n) : GraphElement(GraphStyle::NODE), graph(g.graph) { name = n; SetLabelFromName();}
 public:
     Graph &graph;               ///<The graph we belong to.
 
@@ -401,13 +401,13 @@ inline void Graph::AddAttrName(GraphStyle::EGraphElementType t, const std::strin
 
 
 template <class SG, typename ...Args>
-SubGraph *Graph::CreateSubGraph(const char *name, bool cluster, bool *created, Args &&... args)
+SubGraph *Graph::CreateSubGraph(std::string_view name, bool cluster, bool *created, Args &&... args)
 {
     //Do nothing when a user-re-defines a subgraph - we re-use the same one if it exists among our sisters
     //For unnamed subgraphs: always create a new one
-    auto i = name && name[0] ?
-                 std::find_if(this->subgraphs.begin(), this->subgraphs.end(), [name](auto p) {return p->name==name; }) :
-                 this->subgraphs.end();
+    auto i = name.empty()
+        ? this->subgraphs.end()
+        : std::find_if(this->subgraphs.begin(), this->subgraphs.end(), [name](auto p) {return p->name==name; });
     SubGraph *p = i==this->subgraphs.end() ? nullptr : *i;
     if (created) *created = !p;
     if (!p) {
@@ -420,15 +420,15 @@ SubGraph *Graph::CreateSubGraph(const char *name, bool cluster, bool *created, A
 }
 
 template <class SG, typename ...Args>
-SubGraph *Graph::CreateSubGraph(SubGraph *g, const char *name, bool cluster, bool *created, Args&&... args)
+SubGraph *Graph::CreateSubGraph(SubGraph *g, std::string_view name, bool cluster, bool *created, Args&&... args)
 {
     _ASSERT(g);
     if (g==nullptr) return this->CreateSubGraph<SubGraph>(name, cluster, created, std::forward<Args>(args)...);
     //Do nothing when a user-re-defines a subgraph - we re-use the same one if it exists among our sisters
     //For unnamed subgraphs: always create a new one
-    auto i = name && name[0] ?
-        std::find_if(g->subgraphs.begin(), g->subgraphs.end(), [name](auto p) {return p->name==name; }) :
-        g->subgraphs.end();
+    auto i = name.empty()
+        ? g->subgraphs.end()
+        : std::find_if(g->subgraphs.begin(), g->subgraphs.end(), [name](auto p) {return p->name==name; });
     SubGraph *p = i==g->subgraphs.end() ? nullptr : *i;
     if (created) *created = !p;
     if (!p) {
@@ -441,7 +441,7 @@ SubGraph *Graph::CreateSubGraph(SubGraph *g, const char *name, bool cluster, boo
 }
 
 template <class Node, typename ...Args>
-GraphNode *Graph::CreateNode(const char *name, bool *created, Args &&... args)
+GraphNode *Graph::CreateNode(std::string_view name, bool *created, Args &&... args)
 {
     GraphNode *p = this->GetNode(name);
     if (created) *created = !p;
@@ -459,7 +459,7 @@ GraphNode *Graph::CreateNode(const char *name, bool *created, Args &&... args)
 }
 
 template <class Node, typename ...Args>
-GraphNode *Graph::CreateNode(SubGraph *g, const char *name, bool *created, Args &&... args)
+GraphNode *Graph::CreateNode(SubGraph *g, std::string_view name, bool *created, Args &&... args)
 {
     if (g==nullptr) return CreateNode<Node>(name, created, std::forward<Args>(args)...);
     GraphNode *p = this->GetNode(name);
