@@ -1,6 +1,6 @@
 /*
     This file is part of Msc-generator.
-    Copyright (C) 2008-2022 Zoltan Turanyi
+    Copyright (C) 2008-2023 Zoltan Turanyi
     Distributed under GNU Affero General Public License.
 
     Msc-generator is free software: you can redistribute it and/or modify
@@ -20,6 +20,7 @@
  * @ingroup libmscgen_files */
 
 #include <cmath>
+#include "mscelement.h"
 #include "msc.h"
 
 using namespace msc;
@@ -186,23 +187,23 @@ string Entity::Print(int indent) const
 }
 
 
-EntityAppHelper* EntityAppHelper::Prepend(EntityAppHelper* edh) { 
-    if (edh) { 
-        entities.Prepend(&edh->entities); 
+EntityAppHelper* EntityAppHelper::Prepend(EntityAppHelper* edh) {
+    if (edh) {
+        entities.Prepend(&edh->entities);
         std::ranges::move(notes, std::back_inserter(edh->notes));
         std::swap(notes, edh->notes);
         edh->notes.clear();
         edh->entities.clear();
         //leave "target" as that of the latter edh
-    } 
-    return this; 
-} 
+    }
+    return this;
+}
 
 
 /** Create an EntityApp for entity named `s` onto the chart `chart`
  * Make all attributes unset, empty the style (but activate those style
  * elements that are set in the default entity style in chart->Contexts*/
-EntityApp::EntityApp(const char *s, MscChart* msc) : MscElement(msc),
+EntityApp::EntityApp(std::string_view s, MscChart* msc) : MscElement(msc),
     name(s),
     only_shown_if_multi(),
     show_is_explicit(false),
@@ -221,8 +222,8 @@ EntityApp::EntityApp(const char *s, MscChart* msc) : MscElement(msc),
 /** Create a pseudo Entity App, one that refers to many entities at once.
  * This is used for 'heading' commands, or solo entity prefixes, such as
  * 'show', 'hide', 'activate', 'deactivate'.*/
-EntityApp::EntityApp(MscChart* chart, bool shown_only, const char *prefix, const FileLineCol &l)
-    : MscElement(chart), 
+EntityApp::EntityApp(MscChart* chart, bool shown_only, std::string_view prefix, const FileLineCol &l)
+    : MscElement(chart),
     name(),
     only_shown_if_multi(shown_only),
     show_is_explicit(true),
@@ -361,8 +362,8 @@ bool EntityApp::AddAttribute(const Attribute& a)
  *           other than EntityCommand and Note, in that case we need to give an error.
  * @param [in] l The position of the EntityApp in the file.
  * @return An EntityAppHelper which contains us and our children (if any), plus all notes.
- *         We will be the first EntityApp in the returned list, children will come after. 
- * NOTE: ownership of us (this object) is transferred to the returned object by creating a 
+ *         We will be the first EntityApp in the returned list, children will come after.
+ * NOTE: ownership of us (this object) is transferred to the returned object by creating a
  *   unique ptr from 'this'!*/
 std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al, ArcList *ch, FileLineCol l)
 {
@@ -385,8 +386,8 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
         for (auto &pAttr : *al)
             AddAttribute(*pAttr);
         //Check that if we have a shape and shape.size=auto then the shape has a label box
-        if (auto &s = style.read(); 
-                  s.shape.value_or(-1)>=0 
+        if (auto &s = style.read();
+                  s.shape.value_or(-1)>=0
                && s.shape_size.value_or(EArrowSize::BIG)==EArrowSize::INVALID
                && chart->Shapes[*s.shape].GetLabelPos().IsInvalid()) {
             //Find the (last) shape.size attr
@@ -436,7 +437,7 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
         if (active && active_is_explicit)
             chart->Error.Error(active.file_pos, "You cannot directly activate or deactivate a grouped entity.",
                                "The active/inactive state of grouped entities is derived from its member entities. Ignoring attribute.");
-        active.reset(); 
+        active.reset();
         active_is_explicit = false;
         //consider forced collapse
         auto force_itr = chart->force_entity_collapse.find(name);
@@ -444,7 +445,7 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
             //If force_entity_collapse is same as chart value, remove
             if (collapsed.value_or(false) == force_itr->second)
                 chart->force_entity_collapse.erase(force_itr);
-            else 
+            else
                 collapsed = force_itr->second;
         }
     } else {
@@ -531,7 +532,7 @@ std::unique_ptr<EntityAppHelper> EntityApp::AddAttributeList(AttributeList *al,
 
             //If "entity" style contains no "indicator" value (the default in plain)
             //we use the value from the context (true by default)
-            if (!style_to_use.read().indicator) 
+            if (!style_to_use.read().indicator)
                 style_to_use.write().indicator = chart->MyCurrentContext().indicator.value();
             //Copy shapeand shape size from 'style' (where set by attributes) to 'style_to_use'
             //which contains the default shape.
@@ -686,7 +687,7 @@ void EntityApp::AttributeNames(Csh &csh)
 }
 
 /** Add a list of possible attribute value names to `csh` for attribute `attr`.*/
-bool EntityApp::AttributeValues(const std::string attr, Csh &csh)
+bool EntityApp::AttributeValues(std::string_view attr, Csh &csh)
 {
     if (CaseInsensitiveEqual(attr,"color")) {
         csh.AddColorValuesToHints(false);
@@ -712,7 +713,7 @@ bool EntityApp::AttributeValues(const std::string attr, Csh &csh)
         return true;
     }
     if (CaseInsensitiveEqual(attr, "shape.size")) {
-        csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRVALUE)+"auto", 
+        csh.AddToHints(CshHint(csh.HintPrefix(COLOR_ATTRVALUE)+"auto",
             "Sets the size of the entity as large as needed to accommodate the entity label. "
             "Valid only for shapes with a label placeholder.",
             EHintType::ATTR_VALUE, true));
@@ -767,7 +768,7 @@ double EntityApp::Width() const
     //Entity labels cannot be word wrapped
     double inner = parsed_label.getTextWidthHeight().x;
     if (entity->children_names.size() && style.read().indicator.value() && entity->collapsed)
-        inner = std::max(inner, GetIndiactorSize().x + 2*chart->boxVGapInside);
+        inner = std::max(inner, GetIndicatorSize().x + 2*chart->boxVGapInside);
     const double width = ceil(style.read().line.LineWidth()*2 + inner);
     return width + fmod_negative_safe(width, 2.); //always return an even number
 }
@@ -809,7 +810,7 @@ Range EntityApp::Height(Area &cover, const NPtrList<EntityApp> &children)
         if (entity->children_names.size() && style.read().indicator.value())
             indicator_ypos_offset = wh.y + lw + chart->boxVGapInside;
         const double width = Width();
-        const double indicator_height = (indicator_ypos_offset > 0) ? GetIndiactorSize().y + 2*chart->boxVGapInside : 0;
+        const double indicator_height = (indicator_ypos_offset > 0) ? GetIndicatorSize().y + 2*chart->boxVGapInside : 0;
         const double height = ceil(chart->headingVGapAbove + wh.y + indicator_height + 2*lw + chart->headingVGapBelow);
 
         //do not include shadow in anything... but the returned height (uses for non-compressed placement)
@@ -987,7 +988,7 @@ void EntityApp::Draw(Canvas &canvas)
         do {
             b.y.till = std::min({ entity->status.ShowTill(b.y.till), entity->status.StyleTill(b.y.till), chart->GetTotal().y.till });
         } while (b.y.till < chart->GetTotal().y.till
-                 && entity->status.GetStatus(b.y.till).IsOn() 
+                 && entity->status.GetStatus(b.y.till).IsOn()
                  && compatible(style, entity->status.GetStyle(b.y.till)));
     }
     canvas.Entity(b, parsed_label, style.read(), chart->Shapes, entity->shape, !entity->children_names.empty());
@@ -996,7 +997,7 @@ void EntityApp::Draw(Canvas &canvas)
     if (indicator_ypos_offset > 0)
         DrawIndicator(XY(outer_edge.Centroid().x, outer_edge.y.from + indicator_ypos_offset),
         &canvas);
-    
+
     //For large keep drawing all the span of 'on'-ness
     if (!large) return;
     while (b.y.till < chart->GetTotal().y.till
