@@ -123,7 +123,7 @@ ReadIncludeFile(std::string_view filename, void *, std::string_view included_fro
     return ret;
 }
 
-std::vector<std::string> FileListProc(std::string_view prefix_, std::string_view included_from_, void* app) {
+std::vector<std::string> FileListProc(std::string_view prefix_, std::string_view included_from_) {
     std::string included_from__(included_from_);
     std::string prefix__(prefix_);
     //convert Windows style to UNIX style
@@ -221,6 +221,25 @@ std::optional<std::string> ShowDocumentation(const std::string& doc_name) {
     return {};
 }
 
+std::optional<std::string> ShowLink(std::string_view url) {
+    if (auto [ret, err] = POpen("exec bash -c '"
+                                "for i in xdg-open sensible-browser open; do"
+                                " err=`$i $1 2>&1`;"
+                                " ret=$?;"
+                                " case $ret in"
+                                "  126 | 127) ;;"
+                                "  *) echo $err; exit $ret;;"
+                                " esac;"
+                                "done;"
+                                "echo No documentation browser found;"
+                                "exit 1"
+                                "' msc-doc.sh " + std::string(url));
+    ret)
+        return {err + " (exit status: " + std::to_string(ret) + ')'};
+    return {};
+}
+
+
 #endif
 
 int main(int argc, char* argv[])
@@ -240,13 +259,13 @@ int main(int argc, char* argv[])
 
     //Generate language packs
     LanguageCollection languages;
-    languages.AddLanguage(&MscChart::Factory, FileListProc, nullptr);
-    languages.AddLanguage(&GraphChart::Factory, FileListProc, nullptr);
+    languages.AddLanguage(&MscChart::Factory, FileListProc);
+    languages.AddLanguage(&GraphChart::Factory, FileListProc);
+    languages.AddLanguage(&BlockChart::Factory, FileListProc);
 #ifdef _DEBUG
-    languages.AddLanguage(&XxxChart::Factory, FileListProc, nullptr);
-    languages.AddLanguage(&FlowChart::Factory, FileListProc, nullptr);
+    languages.AddLanguage(&XxxChart::Factory, FileListProc);
+    languages.AddLanguage(&FlowChart::Factory, FileListProc);
 #endif
-    languages.AddLanguage(&BlockChart::Factory, FileListProc, nullptr);
 
     //Load progress balance data
     std::string load_data_fname = hdir + RCDIR_NAME "/" LOAD_FILE_NAME;
@@ -272,14 +291,14 @@ int main(int argc, char* argv[])
     design_files.emplace_back("[designlib].graph", gv_designs);
     design_files.emplace_back("[designlib].block", block_designs);
     for (auto &s : languages.GetExtensions())
-        Search_Read((hdir+RCDIR_NAME "/*."+s).c_str(), design_files);
+        Search_Read(StrCat(hdir, RCDIR_NAME "/*.", s).c_str(), design_files);
 
     std::vector<std::string> args;
     for (int i = 1; i<argc; i++)
         args.emplace_back(argv[i]);
 
 #ifdef BUILD_GUI
-    languages.libraries.merge(RegisterLibrariesGUI());
+    auto gui_libs = RegisterLibrariesGUI();
     SDL_version v_comp, v_link;
     SDL_VERSION(&v_comp);
     SDL_GetVersion(&v_link);
@@ -290,8 +309,9 @@ int main(int argc, char* argv[])
     else
         snprintf(buff, sizeof(buff), "SDL v%d.%d.%d (compiled for: v%d.%d.%d)",
                  v_link.major, v_link.minor, v_link.patch, v_comp.major, v_comp.minor, v_comp.patch);
-    languages.libraries[buff];
-    languages.libraries["OpenGL v3+"];
+    gui_libs[buff];
+    gui_libs["OpenGL v3+"];
+    languages.AddLibraries(std::move(gui_libs));
 
     const bool display_only = getenv("DISPLAY") && getenv("DISPLAY")[0] && !isatty(STDIN_FILENO);
 #else
@@ -299,10 +319,9 @@ int main(int argc, char* argv[])
 #endif //BUILD_GUI
 
     //Do it
-    auto ret = do_main(args, languages, design_files, OpenNamedFile,
-                       ReadIncludeFile, nullptr,
-                       "\\mn(10)", display_only,
-                       progress, NULL, &load_data);
+    auto ret = do_main(args, std::move(languages), std::move(design_files),
+                       OpenNamedFile, ReadIncludeFile, nullptr,
+                       "\\mn(10)", display_only, progress, NULL, &load_data);
     if (std::holds_alternative<int>(ret)) {
         //We have completed a real command-line operation
         //Fallthrough to end
@@ -487,8 +506,8 @@ int main(int argc, char* argv[])
         for (const auto& [name, _] : Documentation)
             doc_names.push_back(name);
 
-        GUIWindowGeometry w = InitGUI(std::get<GUIInit>(ret), 
-                                      settings_dir, std::filesystem::current_path().native(), 
+        GUIWindowGeometry w = InitGUI(std::get<GUIInit>(ret), dpi_mul(bounds),
+                                      settings_dir, std::filesystem::current_path().native(),
                                       example_dirs, std::move(doc_names));
         SDL_DisplayMode DM{0,0,0,0,0};
         SDL_GetCurrentDisplayMode(0, &DM);
@@ -704,7 +723,7 @@ std::pair<ClipboardFormat, std::string> GetClipboard() {
     return {};
 }
 
-bool CopyToClipboard(ClipboardFormat format, const std::string& content, std::string_view type) {
+bool CopyToClipboard(ClipboardFormat /*format*/, const std::string& /*content*/, std::string_view /*type*/) {
     return true; //dont try again
 }
 
